package com.example.myapplication.database.entities.composeui

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.ViewModel
import com.example.myapplication.database.entities.model.Rent
import com.example.myapplication.database.entities.model.RentItemCrossRef
import com.example.myapplication.database.entities.model.Item
import com.example.myapplication.database.entities.model.ItemFromCart
import com.example.myapplication.database.entities.model.UserItemCrossRef
import com.example.myapplication.database.entities.repository.RentRepository
import com.example.myapplication.database.entities.repository.RentItemRepository
import com.example.myapplication.database.entities.repository.UserRepository
import com.example.myapplication.database.entities.repository.UserItemRepository
import kotlinx.coroutines.delay

class CartViewModel(
    private val userItemRepository: UserItemRepository,
    private val rentRepository: RentRepository,
    private val rentItemRepository: RentItemRepository,
    private val userRepository: UserRepository,
) : ViewModel() {
    private val userUid: Int = 1
    var cartUiState by mutableStateOf(CartUiState())
        private set

    suspend fun refreshState() {
        val cart = userRepository.getCartByUser(userUid)
        cartUiState = CartUiState(cart)
    }

    suspend fun addToRent(userId: Int, items: List<ItemFromCart>) {
        if (items.isEmpty())
            return
        val rentId = rentRepository.insertRent(Rent(0, userId))
        items.forEach { item ->
            rentItemRepository.insertRentItem(
                RentItemCrossRef(
                    rentId.toInt(),
                    item.uid,
                    item.weight,
                    item.count
                )
            )
        }
        userItemRepository.deleteUserItems(userId)
        refreshState()
    }

    suspend fun removeFromCart(user: Int, item: Item, count: Int = 1) {
        userItemRepository.deleteUserItem(UserItemCrossRef(user, item.uid, count))
        refreshState()
    }

    suspend fun updateFromCart(userId: Int, item: Item, count: Int, availableCount: Int)
            : Boolean {
        if (count == 0) {
            removeFromCart(userId, item, count)
            return false
        }
        if (count > availableCount)
            return false
        userItemRepository.updateUserItem(UserItemCrossRef(userId, item.uid, count))
        refreshState()
        return true
    }
}

data class CartUiState(val itemList: List<ItemFromCart> = listOf())